<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Twig\Component\Voucher;

use Dedi\SyliusVoucherPlugin\Model\VoucherInterface;
use Sylius\Bundle\UiBundle\Twig\Component\LiveCollectionTrait;
use Sylius\Bundle\UiBundle\Twig\Component\ResourceFormComponentTrait;
use Sylius\Component\Channel\Repository\ChannelRepositoryInterface;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Product\Model\ProductOptionInterface;
use Sylius\Resource\Doctrine\Persistence\RepositoryInterface;
use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\UX\LiveComponent\Attribute\AsLiveComponent;
use Symfony\UX\LiveComponent\Attribute\LiveProp;

#[AsLiveComponent]
class FormComponent
{
    use LiveCollectionTrait;

    /** @use ResourceFormComponentTrait<ProductOptionInterface> */
    use ResourceFormComponentTrait;

    #[LiveProp(hydrateWith: 'hydrateChannel', dehydrateWith: 'dehydrateChannel', fieldName: 'channel')]
    public ?ChannelInterface $channel = null;

    public function __construct(
        RepositoryInterface $repository,
        FormFactoryInterface $formFactory,
        string $resourceClass,
        string $formClass,
        private readonly ChannelRepositoryInterface $channelRepository,
    ) {
        $this->initialize($repository, $formFactory, $resourceClass, $formClass);
    }

    protected function instantiateForm(): FormInterface
    {
        /** @var VoucherInterface $resource */
        $resource = $this->resource;
        $resource->setChannel($this->channel);

        return $this->formFactory->create($this->formClass, $resource);
    }

    public function hydrateChannel(?string $value): ?ChannelInterface
    {
        if (null === $value) {
            return null;
        }

        /** @var ChannelInterface|null $channel */
        $channel = $this->channelRepository->findOneByCode($value);

        return $channel;
    }

    public function dehydrateChannel(?ChannelInterface $channel): ?string
    {
        return $channel?->getCode();
    }
}
